@@ -19,8 +19,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include <pango/pango-ot.h>
 #include "pango-ot-private.h"
+#include "../pango-utils.h"
 
 typedef struct _PangoOTRule PangoOTRule;
 
@@ -59,7 +62,7 @@ pango_ot_ruleset_get_type (void)
       };
       
       object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "PangoOTRuleset",
+                                            I_("PangoOTRuleset"),
                                             &object_info, 0);
     }
   
