@@ -19,6 +19,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include "pangofc-font.h"
 #include "pangofc-fontmap.h"
 #include "pangofc-private.h"
@@ -37,11 +39,10 @@ enum {
 
 typedef struct _PangoFcFontPrivate PangoFcFontPrivate;
 
-#define PANGO_FC_FONT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), PANGO_TYPE_FC_FONT, PangoFcFontPrivate))
-
 struct _PangoFcFontPrivate
 {
   PangoFcDecoder *decoder;
+  gpointer context_key;
 };
 
 static gboolean pango_fc_font_real_has_char  (PangoFcFont *font,
@@ -64,6 +65,13 @@ static PangoFontMetrics *    pango_fc_font_get_metrics  (PangoFont        *font,
 static PangoFontMap *        pango_fc_font_get_font_map (PangoFont        *font);
 static PangoFontDescription *pango_fc_font_describe     (PangoFont        *font);
 
+
+#define PANGO_FC_FONT_LOCK_FACE(font)	(PANGO_FC_FONT_GET_CLASS (font)->lock_face (font))
+#define PANGO_FC_FONT_UNLOCK_FACE(font)	(PANGO_FC_FONT_GET_CLASS (font)->unlock_face (font))
+
+
+
+
 G_DEFINE_TYPE (PangoFcFont, pango_fc_font, PANGO_TYPE_FONT)
 
 static void
@@ -95,6 +103,9 @@ pango_fc_font_class_init (PangoFcFontClass *class)
 static void
 pango_fc_font_init (PangoFcFont *fcfont)
 {
+  fcfont->priv = G_TYPE_INSTANCE_GET_PRIVATE (fcfont,
+					      PANGO_TYPE_FC_FONT,
+					      PangoFcFontPrivate);
 }
 
 static void
@@ -108,7 +119,7 @@ static void
 pango_fc_font_finalize (GObject *object)
 {
   PangoFcFont *fcfont = PANGO_FC_FONT (object);
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (fcfont);
+  PangoFcFontPrivate *priv = fcfont->priv;
 
   g_slist_foreach (fcfont->metrics_by_lang, (GFunc)free_metrics_info, NULL);
   g_slist_free (fcfont->metrics_by_lang);  
@@ -227,7 +238,7 @@ pango_fc_font_get_coverage (PangoFont     *font,
 			    PangoLanguage *language)
 {
   PangoFcFont *fcfont = (PangoFcFont *)font;
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (fcfont);
+  PangoFcFontPrivate *priv = fcfont->priv;
   FcCharSet *charset;
 
   if (priv->decoder)
@@ -273,7 +284,7 @@ static void
 get_face_metrics (PangoFcFont      *fcfont,
 		  PangoFontMetrics *metrics)
 {
-  FT_Face face = pango_fc_font_lock_face (fcfont);
+  FT_Face face = PANGO_FC_FONT_LOCK_FACE (fcfont);
   FcMatrix *fc_matrix;
   FT_Matrix ft_matrix;
   TT_OS2 *os2;
@@ -367,7 +378,7 @@ get_face_metrics (PangoFcFont      *fcfont,
       quantize_position (&metrics->strikethrough_thickness, &metrics->strikethrough_position);
     }
   
-  pango_fc_font_unlock_face (fcfont);
+  PANGO_FC_FONT_UNLOCK_FACE (fcfont);
 }
 
 static int
@@ -502,13 +513,13 @@ pango_fc_font_real_get_glyph (PangoFcFont *font,
   FT_Face face;
   FT_UInt index;
 
-  face = pango_fc_font_lock_face (font);
+  face = PANGO_FC_FONT_LOCK_FACE (font);
   
   index = FcFreeTypeCharIndex (face, wc);
-  if (index > face->num_glyphs)
+  if (index > (FT_UInt)face->num_glyphs)
     index = 0;
 
-  pango_fc_font_unlock_face (font);
+  PANGO_FC_FONT_UNLOCK_FACE (font);
 
   return index;
 }
@@ -530,7 +541,7 @@ pango_fc_font_lock_face (PangoFcFont *font)
 {
   g_return_val_if_fail (PANGO_IS_FC_FONT (font), NULL);
 
-  return PANGO_FC_FONT_GET_CLASS (font)->lock_face (font);
+  return PANGO_FC_FONT_LOCK_FACE (font);
 }
 
 /**
@@ -546,8 +557,8 @@ void
 pango_fc_font_unlock_face (PangoFcFont *font)
 {
   g_return_if_fail (PANGO_IS_FC_FONT (font));
-  
-  PANGO_FC_FONT_GET_CLASS (font)->unlock_face (font);
+   
+  PANGO_FC_FONT_UNLOCK_FACE (font);
 }
 
 /**
@@ -565,7 +576,7 @@ gboolean
 pango_fc_font_has_char (PangoFcFont *font,
 			gunichar     wc)
 {
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (font);
+  PangoFcFontPrivate *priv = font->priv;
   FcCharSet *charset;
 
   g_return_val_if_fail (PANGO_IS_FC_FONT (font), FALSE);
@@ -582,14 +593,14 @@ pango_fc_font_has_char (PangoFcFont *font,
 /**
  * pango_fc_font_get_glyph:
  * @font: a #PangoFcFont
- * @wc: Unicode codepoint to look up
+ * @wc: Unicode character to look up
  * 
- * Gets the glyph index for a given unicode codepoint
+ * Gets the glyph index for a given Unicode character
  * for @font. If you only want to determine
  * whether the font has the glyph, use pango_fc_font_has_char().
  * 
- * Return value: the glyph index, or 0, if the unicode
- *  codepoint doesn't exist in the font.
+ * Return value: the glyph index, or 0, if the Unicode
+ *   character doesn't exist in the font.
  *
  * Since: 1.4
  **/
@@ -597,9 +608,7 @@ PangoGlyph
 pango_fc_font_get_glyph (PangoFcFont *font,
 			 gunichar     wc)
 {
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (font);
-
-  g_return_val_if_fail (PANGO_IS_FC_FONT (font), 0);
+  PangoFcFontPrivate *priv = font->priv;
 
   /* Replace NBSP with a normal space; it should be invariant that
    * they shape the same other than breaking properties.
@@ -666,13 +675,13 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
   g_return_if_fail (PANGO_IS_FC_FONT (font));
   g_return_if_fail (glyphs != NULL);
 
-  face = pango_fc_font_lock_face (font);
+  face = PANGO_FC_FONT_LOCK_FACE (font);
   if (!face)
     return;
 
   if (!FT_HAS_KERNING (face))
     {
-      pango_fc_font_unlock_face (font);
+      PANGO_FC_FONT_UNLOCK_FACE (font);
       return;
     }
   
@@ -688,7 +697,7 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
 	glyphs->glyphs[i-1].geometry.width += PANGO_UNITS_26_6 (kerning.x);
     }
   
-  pango_fc_font_unlock_face (font);
+  PANGO_FC_FONT_UNLOCK_FACE (font);
 }
 
 /**
@@ -705,7 +714,7 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
 PangoFcDecoder *
 _pango_fc_font_get_decoder (PangoFcFont *font)
 {
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (font);
+  PangoFcFontPrivate *priv = font->priv;
 
   return priv->decoder;
 }
@@ -725,7 +734,7 @@ void
 _pango_fc_font_set_decoder (PangoFcFont    *font,
 			    PangoFcDecoder *decoder)
 {
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (font);
+  PangoFcFontPrivate *priv = font->priv;
 
   if (priv->decoder)
     g_object_unref (priv->decoder);
@@ -736,6 +745,23 @@ _pango_fc_font_set_decoder (PangoFcFont    *font,
     g_object_ref (priv->decoder);
 }
 
+gpointer
+_pango_fc_font_get_context_key (PangoFcFont *fcfont)
+{
+  PangoFcFontPrivate *priv = fcfont->priv;
+
+  return priv->context_key;
+}
+
+void
+_pango_fc_font_set_context_key (PangoFcFont *fcfont,
+				gpointer     context_key)
+{
+  PangoFcFontPrivate *priv = fcfont->priv;
+
+  priv->context_key = context_key;
+}
+
 static FT_Glyph_Metrics *
 get_per_char (FT_Face      face,
 	      FT_Int32     load_flags,
@@ -770,6 +796,8 @@ get_per_char (FT_Face      face,
  * The other possibility would be to to make it public in something
  * like it's current form, and also expose glyph information
  * caching functionality similar to pango_ft2_font_set_glyph_info().
+ *
+ * Since: 1.6
  **/
 void
 pango_fc_font_get_raw_extents (PangoFcFont    *fcfont,
@@ -781,7 +809,9 @@ pango_fc_font_get_raw_extents (PangoFcFont    *fcfont,
   FT_Glyph_Metrics *gm;
   FT_Face face;
 
-  face = pango_fc_font_lock_face (fcfont);
+  g_return_if_fail (PANGO_IS_FC_FONT (fcfont));
+
+  face = PANGO_FC_FONT_LOCK_FACE (fcfont);
 
   if (!glyph)
     gm = NULL;
@@ -839,6 +869,6 @@ pango_fc_font_get_raw_extents (PangoFcFont    *fcfont,
 	}
     }
       
-  pango_fc_font_unlock_face (fcfont);
+  PANGO_FC_FONT_UNLOCK_FACE (fcfont);
 }
 
